==> 6.5.3-0-202309260341qtwebengine-everywhere-src-6.5.3.zip: 6.5.3/Src/qtwebengine/src/3rdparty/chromium/weblayer/browser/java/org/chromium/weblayer_private/ExternalNavigationDelegateImpl.java <==
// Copyright 2020 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

package org.chromium.weblayer_private;

import android.content.Context;
import android.content.Intent;
import android.content.pm.ResolveInfo;
import android.os.RemoteException;

import androidx.annotation.Nullable;

import org.chromium.base.Callback;
import org.chromium.base.Function;
import org.chromium.base.supplier.Supplier;
import org.chromium.components.embedder_support.util.UrlUtilities;
import org.chromium.components.external_intents.ExternalNavigationDelegate;
import org.chromium.components.external_intents.ExternalNavigationParams;
import org.chromium.content_public.browser.LoadUrlParams;
import org.chromium.content_public.browser.WebContents;
import org.chromium.ui.base.WindowAndroid;
import org.chromium.url.GURL;
import org.chromium.url.Origin;
import org.chromium.weblayer_private.interfaces.APICallException;
import org.chromium.weblayer_private.interfaces.ExternalIntentInIncognitoUserDecision;

import java.util.List;

/**
 * WebLayer's implementation of the {@link ExternalNavigationDelegate}.
 */
public class ExternalNavigationDelegateImpl implements ExternalNavigationDelegate {
    private final TabImpl mTab;
    private boolean mTabDestroyed;

    public ExternalNavigationDelegateImpl(TabImpl tab) {
        assert tab != null;
        mTab = tab;
    }

    public void onTabDestroyed() {
        mTabDestroyed = true;
    }

    @Override
    public Context getContext() {
        return mTab.getBrowser().getContext();
    }

    @Override
    public boolean willAppHandleIntent(Intent intent) {
        return false;
    }

    @Override
    public boolean shouldDisableExternalIntentRequestsForUrl(GURL url) {
        return false;
    }

    @Override
    public boolean handlesInstantAppLaunchingInternally() {
        return false;
    }

    @Override
    public void dispatchAuthenticatedIntent(Intent intent) {
        // This method should never be invoked in WebLayer as this class always returns false for
        // handlesInstantAppLaunchingInternally().
        assert false;
    }

    @Override
    public boolean shouldAvoidDisambiguationDialog(Intent intent) {
        // Don't show the disambiguation dialog if WebLayer can handle the intent.
        return UrlUtilities.isAcceptedScheme(intent.toUri(0));
    }

    @Override
    public void loadUrlIfPossible(LoadUrlParams loadUrlParams) {
        if (!hasValidTab()) return;
        mTab.loadUrl(loadUrlParams);
    }

    @Override
    public boolean isApplicationInForeground() {
        return mTab.getBrowser().isResumed();
    }

    @Override
    public void maybeSetWindowId(Intent intent) {}

    @Override
    public boolean canLoadUrlInCurrentTab() {
        return true;
    }

    @Override
    public void closeTab() {
        InterceptNavigationDelegateClientImpl.closeTab(mTab);
    }

    @Override
    public boolean isIncognito() {
        return mTab.getProfile().isIncognito();
    }

    @Override
    public boolean hasCustomLeavingIncognitoDialog() {
        return mTab.getExternalIntentInIncognitoCallbackProxy() != null;
    }

    @Override
    public void presentLeavingIncognitoModalDialog(Callback<Boolean> onUserDecision) {
        try {
            mTab.getExternalIntentInIncognitoCallbackProxy().onExternalIntentInIncognito(
                    (Integer result) -> {
                        @ExternalIntentInIncognitoUserDecision
                        int userDecision = result.intValue();
                        switch (userDecision) {
                            case ExternalIntentInIncognitoUserDecision.ALLOW:
                                onUserDecision.onResult(Boolean.valueOf(true));
                                break;
                            case ExternalIntentInIncognitoUserDecision.DENY:
                                onUserDecision.onResult(Boolean.valueOf(false));
                                break;
                            default:
                                assert false;
                        }
                    });
        } catch (RemoteException e) {
            throw new APICallException(e);
        }
    }

    @Override
    public void maybeAdjustInstantAppExtras(Intent intent, boolean isIntentToInstantApp) {}

    @Override
    // This is relevant only if the intent ends up being handled by this app, which does not happen
    // for WebLayer.
    public void maybeSetRequestMetadata(Intent intent, boolean hasUserGesture,
            boolean isRendererInitiated, @Nullable Origin initiatorOrigin) {}

    @Override
    // This is relevant only if the intent ends up being handled by this app, which does not happen
    // for WebLayer.
    public void maybeSetPendingReferrer(Intent intent, GURL referrerUrl) {}

    @Override
    // This is relevant only if the intent ends up being handled by this app, which does not happen
    // for WebLayer.
    public void maybeSetPendingIncognitoUrl(Intent intent) {}

    @Override
    public boolean maybeLaunchInstantApp(GURL url, GURL referrerUrl, boolean isIncomingRedirect,
            boolean isSerpReferrer, Supplier<List<ResolveInfo>> resolveInfoSupplier) {
        return false;
    }

    @Override
    public WindowAndroid getWindowAndroid() {
        return mTab.getBrowser().getWindowAndroid();
    }

    @Override
    public WebContents getWebContents() {
        return mTab.getWebContents();
    }

    @Override
    public boolean hasValidTab() {
        assert mTab != null;
        return !mTabDestroyed;
    }

    @Override
    public boolean canCloseTabOnIncognitoIntentLaunch() {
        return hasValidTab();
    }

    @Override
    public boolean isIntentForTrustedCallingApp(
            Intent intent, Supplier<List<ResolveInfo>> resolveInfoSupplier) {
        return false;
    }

    @Override
    public boolean isIntentToAutofillAssistant(Intent intent) {
        return false;
    }

    @Override
    public @IntentToAutofillAllowingAppResult int isIntentToAutofillAssistantAllowingApp(
            ExternalNavigationParams params, Intent targetIntent,
            Function<Intent, Boolean> canExternalAppHandleIntent) {
        return IntentToAutofillAllowingAppResult.NONE;
    }

    @Override
    public boolean handleWithAutofillAssistant(ExternalNavigationParams params, Intent targetIntent,
            GURL browserFallbackUrl, boolean isGoogleReferrer) {
        return false;
    }

    @Override
    public boolean shouldLaunchWebApksOnInitialIntent() {
        return false;
    }

    @Override
    public boolean maybeSetTargetPackage(
            Intent intent, Supplier<List<ResolveInfo>> resolveInfoSupplier) {
        return false;
    }

    @Override
    public boolean shouldEmbedderInitiatedNavigationsStayInBrowser() {
        // WebLayer already has APIs that allow the embedder to specify that a navigation shouldn't
        // result in an external intent (Navigation#disableIntentProcessing() and
        // NavigateParams#disableIntentProcessing()), and historically embedder-initiated
        // navigations have been allowed to leave the browser on the initial navigation, so we need
        // to maintain that behavior.
        return false;
    }
}

==> 6.7.3-0-202409200836qtwebengine-everywhere-src-6.7.3.zip: 6.7.3/Src/qtwebengine/src/3rdparty/chromium/weblayer/browser/java/org/chromium/weblayer_private/ExternalNavigationDelegateImpl.java <==
// Copyright 2020 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

package org.chromium.weblayer_private;

import android.content.Context;
import android.content.Intent;
import android.content.pm.ResolveInfo;
import android.os.RemoteException;

import org.chromium.base.Callback;
import org.chromium.base.supplier.Supplier;
import org.chromium.components.embedder_support.util.UrlUtilities;
import org.chromium.components.external_intents.ExternalNavigationDelegate;
import org.chromium.content_public.browser.WebContents;
import org.chromium.ui.base.WindowAndroid;
import org.chromium.url.GURL;
import org.chromium.weblayer_private.interfaces.APICallException;
import org.chromium.weblayer_private.interfaces.ExternalIntentInIncognitoUserDecision;

import java.util.List;

/**
 * WebLayer's implementation of the {@link ExternalNavigationDelegate}.
 */
public class ExternalNavigationDelegateImpl implements ExternalNavigationDelegate {
    private final TabImpl mTab;
    private boolean mTabDestroyed;

    public ExternalNavigationDelegateImpl(TabImpl tab) {
        assert tab != null;
        mTab = tab;
    }

    public void onTabDestroyed() {
        mTabDestroyed = true;
    }

    @Override
    public Context getContext() {
        return mTab.getBrowser().getContext();
    }

    @Override
    public boolean willAppHandleIntent(Intent intent) {
        return false;
    }

    @Override
    public boolean shouldDisableExternalIntentRequestsForUrl(GURL url) {
        return !mTab.getBrowser().isExternalIntentsEnabled();
    }

    @Override
    public boolean shouldAvoidDisambiguationDialog(GURL intentDataUrl) {
        // Don't show the disambiguation dialog if WebLayer can handle the intent.
        return UrlUtilities.isAcceptedScheme(intentDataUrl);
    }

    @Override
    public boolean isApplicationInForeground() {
        return mTab.getBrowser().getBrowserFragment().isVisible();
    }

    @Override
    public void maybeSetWindowId(Intent intent) {}

    @Override
    public boolean canLoadUrlInCurrentTab() {
        return true;
    }

    @Override
    public void closeTab() {
        InterceptNavigationDelegateClientImpl.closeTab(mTab);
    }

    @Override
    public boolean isIncognito() {
        return mTab.getProfile().isIncognito();
    }

    @Override
    public boolean hasCustomLeavingIncognitoDialog() {
        return mTab.getExternalIntentInIncognitoCallbackProxy() != null;
    }

    @Override
    public void presentLeavingIncognitoModalDialog(Callback<Boolean> onUserDecision) {
        try {
            mTab.getExternalIntentInIncognitoCallbackProxy().onExternalIntentInIncognito(
                    (Integer result) -> {
                        @ExternalIntentInIncognitoUserDecision
                        int userDecision = result.intValue();
                        switch (userDecision) {
                            case ExternalIntentInIncognitoUserDecision.ALLOW:
                                onUserDecision.onResult(Boolean.valueOf(true));
                                break;
                            case ExternalIntentInIncognitoUserDecision.DENY:
                                onUserDecision.onResult(Boolean.valueOf(false));
                                break;
                            default:
                                assert false;
                        }
                    });
        } catch (RemoteException e) {
            throw new APICallException(e);
        }
    }

    @Override
    // This is relevant only if the intent ends up being handled by this app, which does not happen
    // for WebLayer.
    public void maybeSetRequestMetadata(
            Intent intent, boolean hasUserGesture, boolean isRendererInitiated) {}

    @Override
    // This is relevant only if the intent ends up being handled by this app, which does not happen
    // for WebLayer.
    public void maybeSetPendingReferrer(Intent intent, GURL referrerUrl) {}

    @Override
    // This is relevant only if the intent ends up being handled by this app, which does not happen
    // for WebLayer.
    public void maybeSetPendingIncognitoUrl(Intent intent) {}

    @Override
    public WindowAndroid getWindowAndroid() {
        return mTab.getBrowser().getBrowserFragment().getWindowAndroid();
    }

    @Override
    public WebContents getWebContents() {
        return mTab.getWebContents();
    }

    @Override
    public boolean hasValidTab() {
        assert mTab != null;
        return !mTabDestroyed;
    }

    @Override
    public boolean canCloseTabOnIncognitoIntentLaunch() {
        return hasValidTab();
    }

    @Override
    public boolean isForTrustedCallingApp(Supplier<List<ResolveInfo>> resolveInfoSupplier) {
        return false;
    }

    @Override
    public boolean shouldLaunchWebApksOnInitialIntent() {
        return false;
    }

    @Override
    public void setPackageForTrustedCallingApp(Intent intent) {
        assert false;
    }

    @Override
    public boolean shouldEmbedderInitiatedNavigationsStayInBrowser() {
        // WebLayer already has APIs that allow the embedder to specify that a navigation shouldn't
        // result in an external intent (Navigation#disableIntentProcessing() and
        // NavigateParams#disableIntentProcessing()), and historically embedder-initiated
        // navigations have been allowed to leave the browser on the initial navigation, so we need
        // to maintain that behavior.
        return false;
    }

    @Override
    public String getSelfScheme() {
        return null;
    }
}
